//
//  main.c
//
//  Program main / SANWA PC20 SDcard logger 0.2
//  Copyright (C) 2009 Toyohiko Togashi, suzume-syako.jp
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.02.17 v0.1  First release
//	2009.02.20 v0.2  SDcard write metod, interval intrrupt -> main loop
//

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include <stdio.h>
#include <string.h>

#pragma interrupt_handler RX8_1_ISR

#define RCVSIZE 12
#define WRTSIZE 10
static struct {				// DMM data
	unsigned char dat[14];	// raw data
	DWORD ts;				// time stamp
	WORD tm;				// sec/32768
} rcv[RCVSIZE];
static int rp = 0;			// recieve point
static int cp = 0;			// check point
static int ctr = 0;			// int ctr
static unsigned char fp;	// SD card


//
// DMM data 1byte Recieve
//
void RX8_1_ISR(void)
{
	unsigned char s;
	unsigned char c;

	PWM8_G_Start();

	s = RX8_1_bReadRxStatus();
	c = RX8_1_bReadRxData();

	if (s & RX8_1_RX_NO_ERROR) {
		PWM8_R_Start();

	} else if (s & RX8_1_RX_COMPLETE) {
		unsigned char i;

		PWM8_R_Stop();
		i = c >> 4;
		rcv[rp].dat[i] = c & 0xf;

		// packet termination
		if (i == 0xe) {
			DWORD t;

			// time stamp
			Counter32_1_ReadCounter(&t);
			rcv[rp].ts = Counter32_1_PERIOD - t;
			rcv[rp].tm = Counter16_1_PERIOD - Counter16_1_wReadCounter();

			// next entry
			rp++;
			if (rp >= RCVSIZE) {
				rp = 0;
			}
		}
	}

	PWM8_G_Stop();
}

//
// 7segment to char data
//
unsigned char segconv(int a, int b)
{
	// 7seg patturn
	//
	//  aaa
	// f   b
	// f   b      76543210
	//  ggg   ==> 0efadcgb
	// e   c
	// e   c
	//  ddd
	//
	const int pat[] = { 0x7d, 0x05, 0x5b, 0x1f, 0x27, 0x3e, 0x7e, 0x15, 0x7f, 0x3f, 0x68, 0x6e, 0x35, 0x37, 0x00, 0x80 };
	const int val[] = {  '0',  '1',  '2',  '3',  '4',  '5',  '6',  '7',  '8',  '9',  'L',  '6',  '7',  '9',  ' ',  '*' };
	int i;
	int c;

	c = ((a & 0x7) << 4) | (b & 0x0f);
	for(i = 0; pat[i] < 0x80; i++) {
		if (pat[i] == c) break;
	} 
	return val[i];
}

//
// convert to char data
//
void valconv(unsigned char *buff, char *a, char *s, char *u)
{
	char n[5];		// digit

	// AC/DC
	a[0] = '\0';
	if (buff[1] & 0x08) {
		strcat(a, "AC");
	} else if (buff[1] & 0x04) {
		strcat(a, "DC");
	}

	// 4digit
	n[0] = segconv(buff[2], buff[3]);
	n[1] = segconv(buff[4], buff[5]);
	n[2] = segconv(buff[6], buff[7]);
	n[3] = segconv(buff[8], buff[9]);
	n[4] = '\0';

	// N/A data
	if (n[0] == ' ') {
		strcpy(s, "N/A");

	// normal data
	} else {
		int f;
		int i;
		int j;
		
		// float point
		if (buff[4] & 0x08) {
			f = 0;
		} else if (buff[6] & 0x08) {
			f = 1;
		} else if (buff[8] & 0x08) {
			f = 2;
		} else {
			f = 3;
		}
			
		// zero supprelss
		for(i = 0; i < f; i++) {
			if (n[i] != '0') break;
		}
			
		// add sign
		j = 0;
		if (buff[2] & 0x08) {
			s[j++] = '-';
		}
			
		// insert dot
		for(; i <= 5; i++) {
			s[j++] = n[i];
			if (i == f) {
				if (f < 3) {
					s[j++] = '.';
				}
			}
		}
	}
			
	// unit
	u[0] = '\0';
	if (buff[11] & 0x02) {
		strcat(u, "M");
	} else if (buff[10] & 0x02) {
		strcat(u, "k");
	} else if (buff[11] & 0x08) {
		strcat(u, "m");
	} else if (buff[10] & 0x08) {
		strcat(u, "u");
	} else if (buff[10] & 0x04) {
		strcat(u, "n");
	}

	if (buff[12] & 0x08) {
		strcat(u, "F");
	} else if (buff[12] & 0x04) {
		strcat(u, "Ohm");
	} else if (buff[13] & 0x08) {
		strcat(u, "A");
	} else if (buff[13] & 0x04) {
		strcat(u, "V");
	} else if (buff[13] & 0x02) {
		strcat(u, "Hz");
	}

	return;
}

//
// SD card write
//
void burstwrite() {
	int i; 
	int j;

	PWM8_B_Start();
	j = rp;
	i = cp;
	while(i != j) {
		char t[16];		// TimeStamp
		char a[4];		// AC/DC
		char s[16];		// edited number
		char u[8];		// unit

		// time stamp
		sprintf(t, "%ld.%1ld",
			rcv[i].ts,
			(long)(rcv[i].tm * 10L) / Counter16_1_PERIOD
			);

		// values
		valconv(rcv[i].dat, a, s, u);

		// write
		SDCard_1_fputs(t, fp);		SDCard_1_fputs(",", fp);
		SDCard_1_fputs(a, fp);		SDCard_1_fputs(",", fp);
		SDCard_1_fputs(s, fp);		SDCard_1_fputs(",", fp);
		SDCard_1_fputs(u, fp);		SDCard_1_fputs("\n", fp);
		
		// next
		i++;
		if (i >= RCVSIZE) {
			i = 0;
		}

	}
	SDCard_1_fflush(fp);
	cp = j;

	PWM8_B_Stop();
}

//
// Wait
//
void wait1000() {
	int i;
		
	for(i = 0; i < 3000; i++) {
		int j;

		for (j = 0 ; j < 100; j++);
	}
}

//
//	Main
//
void main()
{
//	Global init
	M8C_EnableGInt;

//	Wait red
	PWM8_R_Start();	wait1000();	PWM8_R_Stop();

//	SD Card init
	SDCard_1_Start();
	SDCard_1_InitCard();

//	Wait green
	PWM8_G_Start();	wait1000();	PWM8_G_Stop();

//	Open & header write
	while ((fp = SDCard_1_fopen("PC20.csv", "a")) == SDCard_1_MAXFILES ) {
		//LED_1_On();
		PWM8_R_Start();
		SDCard_1_InitCard();	// retry
	}
	SDCard_1_fputs("Sec,ACDC,Value,Unit\n", fp);
	SDCard_1_fflush(fp);
	PWM8_R_Stop();

//	Wait blue
	PWM8_B_Start();	wait1000();	PWM8_B_Stop();

//	Serial port init
    DigBuf_1_Start();					// 32KHz driver
    PWM8_1_Start();						// 3/5 prescaler
	RX8_1_Start(RX8_1_PARITY_NONE);		// 2400bps 1stop
	RX8_1_EnableInt();					// serial int start

//	Time clock counter init
	Counter16_1_Start();
	Counter32_1_Start();

//	M8C_EnableIntMask(INT_MSK0, INT_MSK0_SLEEP);	// sleep intervl int start
	while(1) {
		M8C_ClearWDTAndSleep;
		M8C_Sleep;

		// data poring
		{
			int d;

			d = rp - cp;
			if (d < 0) {
				d += RCVSIZE;
			}
			if (d >= WRTSIZE) {
				burstwrite();
			}
		}
	}
}